import type { RxCollectionExtended as RxCollection, RxCollectionCreatorExtended, RxCollectionHooks, RxDbMetadata } from '@ngx-odm/rxdb/config';
import { RxDBService } from '@ngx-odm/rxdb/core';
import { Entity, EntityId } from '@ngx-odm/rxdb/utils';
import type { MangoQuery, RxAttachmentCreator, RxDatabase, RxDatabaseCreator, RxDocument, RxDumpCollection, RxStorageWriteError } from 'rxdb';
import { RxReplicationState } from 'rxdb/plugins/replication';
import { Observable } from 'rxjs';
import { ZoneLike } from './helpers';
export declare class RxDBCollectionService<T extends Entity = {
    id: EntityId;
}> {
    readonly config: RxCollectionCreatorExtended;
    protected readonly dbService: RxDBService;
    protected readonly ngZone: ZoneLike;
    protected readonly currentUrl$: Observable<string>;
    protected readonly updateQueryParamsFn: any;
    private _collection;
    private _replicationState;
    private _init$;
    get initialized$(): Observable<boolean>;
    get collection(): RxCollection<T>;
    get db(): Readonly<RxDatabase>;
    get dbOptions(): Readonly<RxDatabaseCreator>;
    get replicationState(): RxReplicationState<T, unknown> | null;
    get queryParams$(): Observable<MangoQuery<T>>;
    constructor(config: RxCollectionCreatorExtended, dbService: RxDBService, ngZone?: ZoneLike, currentUrl$?: Observable<string>, updateQueryParamsFn?: any);
    destroy(): void;
    setQueryParams(query: MangoQuery<T>): void;
    patchQueryParams(query: MangoQuery<T>): void;
    sync(): Promise<void>;
    info(): Promise<RxDbMetadata>;
    import(docs: T[]): Promise<void>;
    export(): Promise<RxDumpCollection<T>>;
    docs(query?: MangoQuery<T> | Observable<MangoQuery<T>>, withRevAndAttachments?: boolean): Observable<T[]>;
    docsByIds(ids: string[], withRevAndAttachments?: boolean): Observable<T[]>;
    count(query?: MangoQuery<T>): Observable<number>;
    get(id: string, withRevAndAttachments?: boolean): Observable<T | null>;
    insert(data: T): Promise<RxDocument<T>>;
    insertBulk(data: T[]): Promise<{
        success: RxDocument<T>[];
        error: RxStorageWriteError<T>[];
    }>;
    upsert(data: T): Promise<RxDocument<T>>;
    upsertBulk(data: Partial<T>[]): Promise<{
        success: RxDocument<T>[];
        error: RxStorageWriteError<T>[];
    }>;
    set(id: string, data: Partial<T>): Promise<RxDocument<T> | null>;
    updateBulk(query: MangoQuery<T>, data: Partial<T>): Promise<RxDocument<T, {}>[]>;
    remove(entityOrId: T | string): Promise<RxDocument<T> | null>;
    removeBulk(params: string[] | MangoQuery<T>): Promise<{
        success: RxDocument<T>[];
        error: RxStorageWriteError<T>[];
    }>;
    clear(): Promise<void>;
    getAttachments(docId: string): Promise<Blob[] | null>;
    getAttachmentById(docId: string, attachmentId: string): Promise<Blob | null>;
    putAttachment(docId: string, attachment: RxAttachmentCreator): Promise<void>;
    removeAttachment(docId: string, attachmentId: string): Promise<void>;
    addHook<Hook extends RxCollectionHooks>(hook: Hook, handler: Parameters<RxCollection<T>[Hook]>[0], parralel?: boolean): Promise<void>;
    getLocal<L extends Record<string, any>>(id: string): Promise<L | null>;
    getLocal<L extends Record<string, any>, K = keyof L>(id: string, key: K): Promise<L[keyof L] | null>;
    getLocal$<L extends Record<string, any>, K = keyof L>(id: string, key: K): Observable<L[keyof L] | null>;
    getLocal$<L extends Record<string, any>>(id: string): Observable<L | null>;
    insertLocal<L extends object>(id: string, data: L): Promise<void>;
    upsertLocal<L extends object>(id: string, data: L): Promise<void>;
    setLocal<L extends object, K = keyof L>(id: string, prop: K, value: unknown): Promise<void>;
    removeLocal(id: string): Promise<void>;
    private init;
}
