import type { MangoQuery, RxCollection, RxCollectionCreator, RxDatabaseCreator, RxJsonSchema, RxPlugin } from 'rxdb';
import { RxReplicationState } from 'rxdb/plugins/replication';
import { Observable } from 'rxjs';
import type { Merge, SetOptional, SetRequired } from 'type-fest';
export interface RxCollectionCreatorOptions<T = any> {
    schemaUrl?: string;
    initialDocs?: T[];
    recreate?: boolean;
    useQueryParams?: boolean;
    replicationStateFactory?: (col: RxCollection<T>) => RxReplicationState<T, any> | null;
}
export type RxCollectionCreatorExtended<T = any> = Merge<RxCollectionCreator<T>, {
    schema: RxJsonSchema<T>;
    name: string;
    options?: RxCollectionCreatorOptions<T>;
}>;
export type RxCollectionExtended<T = any> = RxCollection<T> & RxCollectionWithMetadata & RxCollectionWiithQueryParams<T>;
export type RxCollectionWiithQueryParams<T = any> = {
    queryParamsInit?: (currentUrl$: Observable<string>, updateQueryParamsInLocationFn: (queryParams: MangoQueryParams) => Promise<any>) => void;
    queryParams?: {
        $: Observable<MangoQuery<T>>;
        get(): MangoQuery<T>;
        set(query: MangoQuery<T>): void;
        patch(query: MangoQuery<T>): void;
    };
};
export type RxCollectionWithMetadata = {
    getMetadata: () => Promise<RxDbMetadata>;
};
export interface RxDbMetadata {
    id: string;
    collectionName: string;
    databaseName: string;
    storageName: string;
    last_modified: number;
    rev: number;
    isFirstTimeInstantiated: boolean;
}
export type RxCollectionHooks = 'preInsert' | 'preSave' | 'preRemove' | 'postInsert' | 'postSave' | 'postRemove' | 'postCreate';
export type MangoQueryParams = {
    selector?: string | undefined;
    sort?: string | undefined;
    skip?: number | undefined;
    limit?: number | undefined;
};
interface NgxRxdbConfigOptions {
    schemas?: Record<string, RxCollectionCreatorExtended>;
    storageType: 'dexie' | 'memory';
    storageOptions?: {};
    dumpPath?: string;
    useQueryParams?: boolean;
    plugins?: RxPlugin[];
}
type RxDatabaseCreatorPartialStorage = SetOptional<RxDatabaseCreator, 'storage'>;
type RxDatabaseCreatorRequireOptions = SetRequired<RxDatabaseCreatorPartialStorage, 'options'>;
export type RxDatabaseCreatorExtended = Merge<RxDatabaseCreatorRequireOptions, {
    options: NgxRxdbConfigOptions;
}>;
export declare function getRxDatabaseCreator(config: RxDatabaseCreatorExtended): RxDatabaseCreator;
export declare const DEFAULT_BACKOFF_FN: (delay: number) => number;
export declare const DEFAULT_LOCAL_DOCUMENT_ID = "local";
export {};
