import { RxDBPreparePlugin, prepareCollections } from '@ngx-odm/rxdb/prepare';
import { NgxRxdbUtils } from '@ngx-odm/rxdb/utils';
import { addRxPlugin, createRxDatabase } from 'rxdb';
import { RxDBPUseQueryParamsPlugin } from '@ngx-odm/rxdb/query-params';
import { RxDBCleanupPlugin } from 'rxdb/plugins/cleanup';
import { RxDBJsonDumpPlugin } from 'rxdb/plugins/json-dump';
import { RxDBLocalDocumentsPlugin } from 'rxdb/plugins/local-documents';
import { RxDBMigrationPlugin } from 'rxdb/plugins/migration-schema';
import { RxDBUpdatePlugin } from 'rxdb/plugins/update';

const { logger: logger$1 } = NgxRxdbUtils;
async function loadRxDBPlugins(plugins = []) {
    try {
        addRxPlugin(RxDBLocalDocumentsPlugin);
        addRxPlugin(RxDBJsonDumpPlugin);
        addRxPlugin(RxDBMigrationPlugin);
        addRxPlugin(RxDBUpdatePlugin);
        addRxPlugin(RxDBCleanupPlugin);
        addRxPlugin(RxDBPreparePlugin);
        addRxPlugin(RxDBPUseQueryParamsPlugin);
        for (const plugin of plugins) {
            addRxPlugin(plugin);
        }
        logger$1.log('rxdb plugins loaded');
    }
    catch (error) {
        throw new Error(error.message ?? error);
    }
}

const { logger } = NgxRxdbUtils;
class RxDBService {
    get db() {
        return this.dbInstance;
    }
    get dbOptions() {
        return this.options;
    }
    get collections() {
        return this.db.collections;
    }
    async destroyDb() {
        try {
            await this.db.remove();
            await this.db.destroy();
            this.dbInstance = null;
            logger.log(`database destroy`);
        }
        catch {
            logger.log(`database destroy error`);
        }
    }
    async initDb(config) {
        if (this.dbInstance) {
            return;
        }
        try {
            await loadRxDBPlugins(config.options?.plugins);
            this.dbInstance = await createRxDatabase(config);
            this.options = config;
            logger.log(`created database "${this.db.name}" with config "${JSON.stringify(config)}"`);
            if (config?.options?.schemas) {
                const bulk = await this.initCollections(config.options.schemas);
                logger.log(`created ${Object.keys(bulk).length} collections bulk: ${Object.keys(bulk)}`);
            }
        }
        catch (error) {
            logger.log('Error initializing the database:', error);
            throw error;
        }
    }
    async initCollections(colConfigs) {
        try {
            const colCreators = await prepareCollections(colConfigs);
            return await this.db.addCollections(colCreators);
        }
        catch (error) {
            logger.log('Error initializing collection(s)', error);
            throw error;
        }
    }
}

export { RxDBService, loadRxDBPlugins };
//# sourceMappingURL=ngx-odm-rxdb-core.mjs.map
