import * as i0 from '@angular/core';
import { InjectionToken, APP_INITIALIZER, NgZone, inject, NgModule, Optional, SkipSelf, Inject, Self } from '@angular/core';
import * as i2 from '@ngx-odm/rxdb/collection';
import { RxDBCollectionService } from '@ngx-odm/rxdb/collection';
import * as i1 from '@ngx-odm/rxdb/core';
import { RxDBService } from '@ngx-odm/rxdb/core';
import { NgxRxdbUtils } from '@ngx-odm/rxdb/utils';
import { Location } from '@angular/common';
import { Router } from '@angular/router';
import { BehaviorSubject, distinctUntilChanged, defer } from 'rxjs';

const RXDB = new InjectionToken('RxDBService Instance', {
    providedIn: 'root',
    factory: () => new RxDBService(),
});
const RXDB_CONFIG = new InjectionToken('RxDatabaseCreator');
const RXDB_CONFIG_COLLECTION = new InjectionToken('RxCollectionCreator');
const RXDB_COLLECTION = new InjectionToken('RxDBCollectionService');
function dbInitializerFactory(dbService, dbConfig) {
    return async () => {
        await dbService.initDb(dbConfig);
    };
}
function provideRxDatabase(config) {
    return [
        { provide: RXDB_CONFIG, useValue: config },
        {
            provide: APP_INITIALIZER,
            useFactory: dbInitializerFactory,
            deps: [RXDB, RXDB_CONFIG],
            multi: true,
        },
    ];
}
function provideRxCollection(collectionConfig) {
    return [
        { provide: RXDB_CONFIG_COLLECTION, useValue: collectionConfig },
        {
            provide: RXDB_COLLECTION,
            useFactory: (config, dbService, ngZone, currentUrl, updateQueryParamsFn) => new RxDBCollectionService(config, dbService, ngZone, currentUrl, updateQueryParamsFn),
            deps: [
                RXDB_CONFIG_COLLECTION,
                RXDB,
                NgZone,
                CURRENT_URL,
                updateQueryParams,
            ],
        },
    ];
}
const CURRENT_URL = new InjectionToken('CURRENT_URL', {
    providedIn: 'root',
    factory: () => {
        const location = inject(Location);
        const subject = new BehaviorSubject(window.location.href);
        location.onUrlChange(url => {
            subject.next(url);
        });
        return defer(() => subject).pipe(distinctUntilChanged());
    },
});
const updateQueryParams = new InjectionToken('UPDATE_QUERY_PARAMS', {
    providedIn: 'root',
    factory: () => {
        const router = inject(Router);
        return (queryParams) => {
            return router.navigate([], {
                queryParams,
                queryParamsHandling: 'merge',
            });
        };
    },
});

class NgxRxdbModule {
    static forFeature(collectionConfig) {
        return {
            ngModule: NgxRxdbFeatureModule,
            providers: provideRxCollection(collectionConfig),
        };
    }
    static forRoot(config) {
        return {
            ngModule: NgxRxdbModule,
            providers: provideRxDatabase(config),
        };
    }
    constructor(appInitStatus, ngxRxdbConfig, trueNgxRxdbConfig, injector) {
        if (!trueNgxRxdbConfig && !ngxRxdbConfig) {
            throw new Error(`${RXDB_CONFIG.toString()} is not provided. Make sure you call the 'forRoot' method of the NgxRxdbModule in the AppModule only.`);
        }
        if (trueNgxRxdbConfig && ngxRxdbConfig) {
            throw new Error(`${RXDB_CONFIG.toString()} is already provided. Make sure you call the 'forRoot' method of the NgxRxdbModule in the AppModule only.`);
        }
        if (trueNgxRxdbConfig && !ngxRxdbConfig) {
            appInitStatus.donePromise.then(() => {
                const ngxRxdbService = injector.get(RXDB);
                if (ngxRxdbService.db.startupErrors.length) {
                    NgxRxdbUtils.logger.log(ngxRxdbService.db.startupErrors);
                }
                NgxRxdbUtils.logger.log(`database "${ngxRxdbService.db.name}" ready, rxdb version is "${ngxRxdbService.db['rxdbVersion']}", storage is "${ngxRxdbService.db.storage.name}"`);
            });
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: NgxRxdbModule, deps: [{ token: i0.ApplicationInitStatus }, { token: RXDB_CONFIG, optional: true, skipSelf: true }, { token: RXDB_CONFIG, optional: true, self: true }, { token: i0.Injector }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.0.8", ngImport: i0, type: NgxRxdbModule }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: NgxRxdbModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: NgxRxdbModule, decorators: [{
            type: NgModule
        }], ctorParameters: () => [{ type: i0.ApplicationInitStatus }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [RXDB_CONFIG]
                }] }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Self
                }, {
                    type: Inject,
                    args: [RXDB_CONFIG]
                }] }, { type: i0.Injector }] });
class NgxRxdbFeatureModule {
    constructor(dbService, collectionService) {
        this.dbService = dbService;
        this.collectionService = collectionService;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: NgxRxdbFeatureModule, deps: [{ token: RXDB }, { token: RXDB_COLLECTION }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.0.8", ngImport: i0, type: NgxRxdbFeatureModule }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: NgxRxdbFeatureModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: NgxRxdbFeatureModule, decorators: [{
            type: NgModule
        }], ctorParameters: () => [{ type: i1.RxDBService, decorators: [{
                    type: Inject,
                    args: [RXDB]
                }] }, { type: i2.RxDBCollectionService, decorators: [{
                    type: Inject,
                    args: [RXDB_COLLECTION]
                }] }] });

export { CURRENT_URL, NgxRxdbFeatureModule, NgxRxdbModule, RXDB, RXDB_COLLECTION, RXDB_CONFIG, RXDB_CONFIG_COLLECTION, provideRxCollection, provideRxDatabase, updateQueryParams };
//# sourceMappingURL=ngx-odm-rxdb.mjs.map
