import { Signal } from '@angular/core';
import { SignalStoreFeature } from '@ngrx/signals';
import { NamedEntitySignals } from '@ngrx/signals/entities/src/models';
import { SignalStoreFeatureResult } from '@ngrx/signals/src/signal-store-models';
import { Entity, EntityId } from '@ngx-odm/rxdb/utils';
import type { MangoQuery, MangoQuerySelector, MangoQuerySelectorAndIndex } from 'rxdb';
export type Filter = string | Record<string, unknown>;
export type QuerySelect<T = Entity> = {
    selector?: MangoQuerySelector<T>;
};
export declare function getCollectionServiceKeys(options: {
    collection?: string;
}): {
    filterKey: string;
    setQueryParamsKey: string;
    updateQueryParamsKey: string;
    selectedIdsKey: string;
    selectedEntitiesKey: string;
    restoreFilterKey: string;
    updateFilterKey: string;
    updateSelectedKey: string;
    findKey: string;
    entitiesKey: string;
    entityMapKey: string;
    idsKey: string;
    currentKey: string;
    setCurrentKey: string;
    insertKey: string;
    updateKey: string;
    updateAllByKey: string;
    removeKey: string;
    removeAllByKey: string;
};
export type NamedCollectionServiceState<E extends Entity, F extends Filter, CName extends string> = {
    [K in CName as `${K}Filter`]: F;
} & {
    [`query`]: MangoQuery<E>;
} & {
    [K in CName as `selected${Capitalize<K>}Ids`]: Record<EntityId, boolean>;
} & {
    [K in CName as `current${Capitalize<K>}`]: E;
};
export type CollectionServiceState<E extends Entity, F extends Filter> = {
    filter: F;
    query: MangoQuery<E>;
    selectedIds: Record<EntityId, boolean>;
    current: E;
};
export type NamedCollectionServiceSignals<E extends Entity, CName extends string> = {
    [K in CName as `selected${Capitalize<K>}Entities`]: Signal<E[]>;
} & {
    [K in CName as `count${Capitalize<K>}Entities`]: Signal<number>;
} & {
    [K in CName as `countAll`]: Signal<number>;
} & {
    [K in CName as `countFiltered`]: Signal<number>;
};
export type CollectionServiceSignals<E extends Entity> = {
    selectedEntities: Signal<E[]>;
    count: Signal<number>;
    countAll: Signal<number>;
    countFiltered: Signal<number>;
};
export type NamedCollectionServiceMethods<E extends Entity, F extends Filter, CName extends string> = {
    [`sync`]: () => void;
} & {
    [K in CName as `update${Capitalize<K>}Filter`]: (filter: F) => void;
} & {
    [K in CName as `setQueryParams`]: (filter: F) => void;
} & {
    [K in CName as `updateQueryParams`]: (filter: F) => void;
} & {
    [K in CName as `updateSelected${Capitalize<K>}Entities`]: (id: EntityId, selected: boolean) => void;
} & {
    [K in CName as `setCurrent${Capitalize<K>}`]: (entity: E) => void;
} & {
    [K in CName as `find${Capitalize<K>}ById`]: (id: EntityId) => Promise<void>;
} & {
    [K in CName as `insert${Capitalize<K>}`]: (entity: E) => Promise<void>;
} & {
    [K in CName as `update${Capitalize<K>}`]: (entity: E) => Promise<void>;
} & {
    [K in CName as `updateAll${Capitalize<K>}By`]: (query: QuerySelect<E>, data: Partial<E>) => Promise<void>;
} & {
    [K in CName as `remove${Capitalize<K>}`]: (entity: E) => Promise<void>;
} & {
    [K in CName as `removeAll${Capitalize<K>}By`]: (query: QuerySelect<E>) => Promise<void>;
};
export type CollectionServiceMethods<E extends Entity, F extends Filter> = {
    sync: () => Promise<void>;
    updateFilter: (filter: F) => void;
    setQueryParams(query: MangoQuery<E>): void;
    updateQueryParams(query: MangoQuery<E>): void;
    updateSelected: (id: EntityId, selected: boolean) => void;
    setCurrent(entity: E): void;
    insert(entity: E): Promise<void>;
    update(entity: E): Promise<void>;
    updateAllBy(query: QuerySelect<E>, data: Partial<E>): Promise<void>;
    remove(entity: E): Promise<void>;
    removeAllBy(query: QuerySelect<E>): Promise<void>;
};
export declare function withCollectionService<E extends Entity, F extends Filter, CName extends string>(options: {
    collection: CName;
    filter: F;
    query?: MangoQuery<E>;
    countQuery?: MangoQuerySelectorAndIndex<E>;
}): SignalStoreFeature<{
    state: Record<string, never>;
    signals: NamedEntitySignals<E, CName>;
    methods: Record<string, never>;
}, {
    state: NamedCollectionServiceState<E, F, CName>;
    signals: NamedCollectionServiceSignals<E, CName>;
    methods: NamedCollectionServiceMethods<E, F, CName>;
}>;
export declare function withCollectionService<E extends Entity, F extends Filter>(options: {
    filter: F;
    query?: MangoQuery<E>;
    countQuery?: MangoQuerySelectorAndIndex<E>;
}): SignalStoreFeature<SignalStoreFeatureResult, {
    state: CollectionServiceState<E, F>;
    signals: CollectionServiceSignals<E>;
    methods: CollectionServiceMethods<E, F>;
}>;
