import type { FilledMangoQuery, PreparedQuery } from 'rxdb';
import { RxReplicationState } from 'rxdb/plugins/replication';
import { Observable, OperatorFunction, Subject } from 'rxjs';
export type EntityId = string;
export type Entity = {
    id: EntityId;
};
export declare namespace NgxRxdbUtils {
    function clone<T>(value: T): T;
    function keys<T>(object: Nil | T): Array<StringifiedKey<T>>;
    function keysOfNonArray<T>(object: Nil | T): Array<StringifiedKey<T>>;
    function forOwn<T>(object: T, iteratee: ObjectIteratee<T, boolean | void>): T;
    function forOwnOfNonArray<T>(object: T, iteratee: ObjectIteratee<T, boolean | void>): T;
    function forEach<T extends Nil | readonly any[]>(array: T, iteratee: ArrayIteratee<NonNullable<T>[number], boolean | void>): T;
    function forEach<T>(object: T, iteratee: ObjectIteratee<NonNullable<T>, boolean | void>): T;
    function forEachOfArray<T>(array: readonly T[], iteratee: ArrayIteratee<T, boolean | void>): void;
    function merge<A extends object, B extends object>(object: A, source: B): A & B;
    function merge<A extends object, B extends object, C extends object>(object: A, source1: B, source2: C): A & B & C;
    function isEmpty(value: any): boolean;
    const isFunction: (value: any) => value is Function;
    const isUndefined: (value: any) => value is undefined;
    function isNullOrUndefined(value: any): value is Nil;
    const isObject: (x: any) => x is object;
    const isEmptyObject: (x: any) => x is object;
    function isDevMode(): boolean;
    function isValidNumber(value: any): value is number;
    function noop(): void;
    function identity<T>(value: T): T;
    function times<T>(n: number, iteratee: (index: number) => T): T[];
    function range(end: number): number[];
    function range(start: number, end: number, step?: number): number[];
    function getMaybeId(entityOrId: object | string): string;
    function compact<T>(array: Array<T>): Array<Exclude<T, Falsy>>;
    function compactObject<T extends object>(obj: T): Partial<T>;
    function isDevModeForced(): boolean;
    function isTestEnvironment(): boolean;
    const logger: {
        log: (...args: any[]) => void;
        table: {
            (tabularData?: any, properties?: string[] | undefined): void;
            (tabularData: any, properties?: readonly string[] | undefined): void;
        };
    };
    function debug<T>(tag?: string): OperatorFunction<T, T>;
    function retryWithBackoff<T>(count?: number, retryTime?: number): (obs$: Observable<T>) => Observable<T>;
    const tapOnce: <T>(callback: () => void) => (source: Observable<T>) => Observable<T>;
    const deferUntil: (subj: Subject<any>) => (source: Observable<any>) => Observable<any>;
    const getDefaultQuery: () => FilledMangoQuery<any>;
    const getDefaultPreparedQuery: () => PreparedQuery<any>;
}
export declare const getDefaultFetch: () => typeof fetch;
export declare function getDefaultFetchWithHeaders(headers?: Record<string, string>): (url: string, options?: Record<string, any>) => Promise<Response>;
export declare function isValidRxReplicationState<T>(obj: any): obj is RxReplicationState<T, unknown>;
export declare function mapFindResultToJsonArray(withRevAndAttachments?: boolean): OperatorFunction<any[] | Map<string, any>, any[]>;
